@@ -6,9 +6,9 @@ from django.conf import settings
 from dojango.util import media
 
 urlpatterns = patterns('dojango',
-    (r'^test/$', 'views.test'),
-    (r'^test/countries/$', 'views.test_countries'),
-    (r'^test/states/$', 'views.test_states'),
+    url(r'^test/$', 'views.test', name='dojango-test'),
+    url(r'^test/countries/$', 'views.test_countries'),
+    url(r'^test/states/$', 'views.test_states'),
     # Note: define accessible objects in DOJANGO_DATAGRID_ACCESS setting
     url(r'^datagrid-list/(?P<app_name>.+)/(?P<model_name>.+)/$', 'views.datagrid_list', name="dojango-datagrid-list"),
 )
